from .encoder import Encoder
from .functions import base_to_bytes, bytes_to_base

__all__ = ["ALPHABET", "Base94Encoder"]

ALPHABET = "".join([chr(i) for i in range(33, 127)])


class Base94Encoder(Encoder):
    @staticmethod
    def encode(data: bytes) -> str:
        return bytes_to_base(data, ALPHABET)

    @staticmethod
    def decode(data: str) -> bytes:
        return base_to_bytes(data, ALPHABET)



import string
from typing import TYPE_CHECKING, NamedTuple

from rtty_soda.encoders import Encoder, encode_str

if TYPE_CHECKING:
    from rtty_soda.formatters import Formatter
    from rtty_soda.interfaces import Reader, Writer

__all__ = ["FormattedData", "Service"]


class FormattedData(NamedTuple):
    data: bytes
    length: int  # data len in chars or in bytes
    groups: int  # number of groups


class Service:
    def __init__(
        self, formatter: Formatter | None, writer: Writer, verbose: bool
    ) -> None:
        self.formatter = formatter
        self.writer = writer
        self.verbose = verbose

    @staticmethod
    def remove_whitespace(data: str) -> str:
        trans = str.maketrans("", "", string.whitespace)
        return data.translate(trans)

    def read_input(self, source: Reader, encoder: Encoder | None) -> bytes:
        """Read key or ciphertext."""
        if encoder is None:
            return source.read_bytes()

        data = source.read_str()
        data = self.remove_whitespace(data)
        return encoder.decode(data)

    def format_data(self, data: bytes, encoder: Encoder | None) -> FormattedData:
        """Format key or ciphertext."""
        if encoder is None or self.formatter is None:
            return FormattedData(data, len(data), 1)

        data_str = encoder.encode(data)
        formatted = self.formatter.format(data_str)
        data_bytes = encode_str(formatted.text)
        return FormattedData(data_bytes, len(data_str), formatted.groups)

    def write_output(self, data: bytes, encoder: Encoder | None) -> None:
        """Write key or ciphertext."""
        buff = self.format_data(data, encoder)
        self.writer.write_bytes(buff.data)
        if self.verbose:
            self.writer.write_diag(f"Length: {buff.length}")
            self.writer.write_diag(f"Groups: {buff.groups}")



from typing import Protocol

__all__ = ["Reader", "Writer"]


class Reader(Protocol):
    """Reads data source."""

    def read_str(self) -> str: ...

    def read_bytes(self) -> bytes: ...


class Writer(Protocol):
    """Outputs data and prints diagnostics."""

    def write_bytes(self, data: bytes) -> None: ...

    def write_diag(self, message: str) -> None: ...



from pathlib import Path
from typing import TYPE_CHECKING

import click

if TYPE_CHECKING:
    from click.decorators import FC

__all__ = ["CliOptions", "out_path"]

out_path = click.Path(
    file_okay=True, dir_okay=False, writable=True, allow_dash=True, path_type=Path
)


class CliOptions:
    @staticmethod
    def compression(function: FC) -> FC:
        return click.option(
            "--compression",
            "-c",
            default="brotli",
            show_default=True,
            envvar="SODA_COMPRESSION",
        )(function)

    @staticmethod
    def data_encoding(function: FC) -> FC:
        return click.option(
            "--data-encoding",
            "-e",
            default="base64",
            show_default=True,
            envvar="SODA_DATA_ENCODING",
        )(function)

    @staticmethod
    def key_encoding(function: FC) -> FC:
        return click.option(
            "--key-encoding",
            default="base64",
            show_default=True,
            envvar="SODA_KEY_ENCODING",
        )(function)

    @staticmethod
    def short_key_encoding(function: FC) -> FC:
        return click.option(
            "--encoding",
            "-e",
            default="base64",
            show_default=True,
            envvar="SODA_KEY_ENCODING",
        )(function)

    @staticmethod
    def kdf_profile(function: FC) -> FC:
        return click.option(
            "--kdf-profile",
            "-p",
            default="sensitive",
            show_default=True,
            envvar="SODA_KDF_PROFILE",
        )(function)

    @staticmethod
    def short_kdf_profile(function: FC) -> FC:
        return click.option(
            "--profile",
            "-p",
            default="sensitive",
            show_default=True,
            envvar="SODA_KDF_PROFILE",
        )(function)

    @staticmethod
    def verbose(function: FC) -> FC:
        return click.option(
            "--verbose",
            "-v",
            is_flag=True,
            envvar="SODA_VERBOSE",
            help="Show verbose output.",
        )(function)

    @staticmethod
    def text(function: FC) -> FC:
        return click.option(
            "--text",
            "-t",
            is_flag=True,
            envvar="SODA_TEXT",
            help="Treat message as text (binary if not specified).",
        )(function)

    @staticmethod
    def group_len(function: FC) -> FC:
        return click.option(
            "--group-len", default=0, show_default=True, envvar="SODA_GROUP_LEN"
        )(function)

    @staticmethod
    def line_len(function: FC) -> FC:
        return click.option(
            "--line-len", default=80, show_default=True, envvar="SODA_LINE_LEN"
        )(function)

    @staticmethod
    def output_file(function: FC) -> FC:
        return click.option(
            "--output-file", "-o", type=out_path, help="Write output to file."
        )(function)

    @staticmethod
    def padding(function: FC) -> FC:
        return click.option(
            "--padding", default=0, show_default=True, envvar="SODA_PADDING"
        )(function)



from pathlib import Path

from click import Command

cli: Command

def genkey_cmd(
    encoding: str,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None: ...
def pubkey_cmd(
    private_key_file: Path,
    encoding: str,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None: ...
def kdf_cmd(
    password_file: Path,
    encoding: str,
    profile: str,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None: ...
def encrypt_public_cmd(
    private_key_file: Path,
    public_key_file: Path,
    message_file: Path,
    text: bool,
    key_encoding: str,
    data_encoding: str,
    compression: str,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None: ...
def encrypt_secret_cmd(
    key_file: Path,
    message_file: Path,
    text: bool,
    key_encoding: str,
    data_encoding: str,
    compression: str,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None: ...
def encrypt_password_cmd(
    password_file: Path,
    message_file: Path,
    text: bool,
    kdf_profile: str,
    data_encoding: str,
    compression: str,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None: ...
def decrypt_public_cmd(
    private_key_file: Path,
    public_key_file: Path,
    message_file: Path,
    text: bool,
    key_encoding: str,
    data_encoding: str,
    compression: str,
    output_file: Path | None,
) -> None: ...
def decrypt_secret_cmd(
    key_file: Path,
    message_file: Path,
    text: bool,
    key_encoding: str,
    data_encoding: str,
    compression: str,
    output_file: Path | None,
) -> None: ...
def decrypt_password_cmd(
    password_file: Path,
    message_file: Path,
    text: bool,
    kdf_profile: str,
    data_encoding: str,
    compression: str,
    output_file: Path | None,
) -> None: ...
def encode_cmd(
    in_encoding: str,
    out_encoding: str,
    file: Path,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None: ...



from typing import TYPE_CHECKING

from nacl.public import PrivateKey, PublicKey

from rtty_soda.archivers import ARCHIVERS, UNARCHIVERS
from rtty_soda.cryptography import public, secret
from rtty_soda.encoders import ENCODERS, encode_str, scsu_decode, scsu_encode

from .key_service import KeyService
from .service import Service

if TYPE_CHECKING:
    from collections.abc import Callable

    from rtty_soda.formatters import Formatter
    from rtty_soda.interfaces import Reader, Writer

__all__ = ["EncryptionService", "Keypair", "Pipe"]

type Keypair = tuple[PrivateKey, PublicKey]
type Pipe = Callable[[bytes], bytes]


class EncryptionService(Service):
    def __init__(
        self,
        text_mode: bool,
        key_encoding: str,
        data_encoding: str,
        compression: str,
        formatter: Formatter | None,
        writer: Writer,
        verbose: bool,
    ) -> None:
        super().__init__(formatter, writer, verbose)
        self.text_mode = text_mode
        self.key_encoder = ENCODERS.get(key_encoding)
        self.data_encoder = ENCODERS.get(data_encoding)
        self.archiver = ARCHIVERS.get(compression)
        self.unarchiver = UNARCHIVERS.get(compression)

    def encryption_flow(self, message: Reader, encrypt: Pipe) -> None:
        if self.text_mode:
            text = message.read_str().strip()
            plaintext_len = len(text)
            data = scsu_encode(text)
        else:
            data = message.read_bytes()
            plaintext_len = len(data)

        if self.archiver is not None:
            data = self.archiver(data)

        data = encrypt(data)
        buff = self.format_data(data, self.data_encoder)
        writer = self.writer
        writer.write_bytes(buff.data)
        if self.verbose:
            overhead = buff.length / plaintext_len
            writer.write_diag(f"Plaintext: {plaintext_len}")
            writer.write_diag(f"Ciphertext: {buff.length}")
            writer.write_diag(f"Overhead: {overhead:.3f}")
            writer.write_diag(f"Groups: {buff.groups}")

    def read_keypair(self, private_key: Reader, public_key: Reader) -> Keypair:
        priv_bytes = self.read_input(private_key, self.key_encoder)
        priv = PrivateKey(priv_bytes)
        pub_bytes = self.read_input(public_key, self.key_encoder)
        pub = PublicKey(pub_bytes)
        return priv, pub

    def encrypt_public(
        self, private_key: Reader, public_key: Reader, message: Reader
    ) -> None:
        priv, pub = self.read_keypair(private_key, public_key)

        def encrypt(data: bytes) -> bytes:
            return public.encrypt(private=priv, public=pub, data=data)

        self.encryption_flow(message, encrypt)

    def encrypt_secret(self, key: Reader, message: Reader) -> None:
        key_bytes = self.read_input(key, self.key_encoder)

        def encrypt(data: bytes) -> bytes:
            return secret.encrypt(key=key_bytes, data=data)

        self.encryption_flow(message, encrypt)

    def encrypt_password(
        self, password: Reader, message: Reader, kdf_profile: str
    ) -> None:
        key = KeyService.derive_key(password, kdf_profile)

        def encrypt(data: bytes) -> bytes:
            return secret.encrypt(key, data)

        self.encryption_flow(message, encrypt)

    def decryption_flow(self, message: Reader, decrypt: Pipe) -> None:
        data = self.read_input(message, self.data_encoder)
        data = decrypt(data)
        if self.unarchiver is not None:
            data = self.unarchiver(data)

        if self.text_mode:
            data = encode_str(scsu_decode(data))

        self.writer.write_bytes(data)

    def decrypt_public(
        self, private_key: Reader, public_key: Reader, message: Reader
    ) -> None:
        priv, pub = self.read_keypair(private_key, public_key)

        def decrypt(data: bytes) -> bytes:
            return public.decrypt(private=priv, public=pub, data=data)

        self.decryption_flow(message, decrypt)

    def decrypt_secret(self, key: Reader, message: Reader) -> None:
        key_bytes = self.read_input(key, self.key_encoder)

        def decrypt(data: bytes) -> bytes:
            return secret.decrypt(key=key_bytes, data=data)

        self.decryption_flow(message, decrypt)

    def decrypt_password(
        self, password: Reader, message: Reader, kdf_profile: str
    ) -> None:
        key = KeyService.derive_key(password, kdf_profile)

        def decrypt(data: bytes) -> bytes:
            return secret.decrypt(key, data)

        self.decryption_flow(message, decrypt)



from scsu.codecs.decoder import SCSUIncrementalDecoder
from scsu.codecs.encoder import SCSUIncrementalEncoder

__all__ = ["scsu_decode", "scsu_encode"]


def scsu_encode(data: str) -> bytes:
    encoder = SCSUIncrementalEncoder(errors="strict")
    return encoder.encode(s=data, final=True)


def scsu_decode(data: bytes) -> str:
    decoder = SCSUIncrementalDecoder(errors="strict")
    return decoder.decode(s=data, final=True)



import math

__all__ = ["base_to_bytes", "bytes_to_base", "decode_bytes", "encode_str"]


def encode_str(data: str) -> bytes:
    return data.encode(encoding="utf-8", errors="strict")


def decode_bytes(data: bytes) -> str:
    return data.decode(encoding="utf-8", errors="strict")


def int_to_base(number: int, alphabet: str) -> str:
    if number == 0:
        return alphabet[0]

    result: list[str] = []
    base = len(alphabet)
    abs_number = abs(number)
    while abs_number:
        abs_number, remainder = divmod(abs_number, base)
        result.append(alphabet[remainder])

    return "".join(reversed(result))


def base_to_int(source: str, alphabet: str) -> int:
    number = 0
    base = len(alphabet)
    for digit in source:
        number = number * base + alphabet.index(digit)

    return number


def bytes_to_base(source: bytes, alphabet: str) -> str:
    number = int.from_bytes(source, byteorder="big", signed=False)
    return int_to_base(number, alphabet)


def int_to_bytes(number: int) -> bytes:
    if number == 0:
        return b"\x00"

    byte_len = math.ceil(number.bit_length() / 8)
    return number.to_bytes(byte_len, byteorder="big", signed=False)


def base_to_bytes(source: str, alphabet: str) -> bytes:
    return int_to_bytes(base_to_int(source, alphabet))



import itertools
import math
from typing import NamedTuple, Protocol

__all__ = ["FixedFormatter", "FormattedText", "Formatter"]


class FormattedText(NamedTuple):
    text: str
    groups: int


class Formatter(Protocol):
    def format(self, text: str) -> FormattedText: ...


class FixedFormatter(Formatter):
    def __init__(self, group_len: int, line_len: int, pad_count: int) -> None:
        self.group_len = group_len
        self.line_len = line_len
        self.pad_count = pad_count

    def format(self, text: str) -> FormattedText:
        groups = 1

        if 0 < self.group_len < self.line_len:
            text, groups = self.split_groups(text)

        if self.pad_count > 0:
            text = self.pad_newlines(text)

        return FormattedText(text, groups)

    def split_groups(self, data: str) -> FormattedText:
        step = self.group_len
        gpl = self.line_len // (step + 1)
        groups = (data[i : i + step] for i in range(0, len(data), step))
        groups_len = math.ceil(len(data) / step)
        lines = (" ".join(ln) for ln in itertools.batched(groups, gpl, strict=False))
        return FormattedText("\n".join(lines), groups_len)

    def pad_newlines(self, text: str) -> str:
        padding = "\n" * self.pad_count
        return padding + text.strip() + "\n" + padding



import base64

from .encoder import Encoder
from .functions import decode_bytes, encode_str

__all__ = ["Base64Encoder"]


class Base64Encoder(Encoder):
    @staticmethod
    def encode(data: bytes) -> str:
        return decode_bytes(base64.b64encode(data))

    @staticmethod
    def decode(data: str) -> bytes:
        return base64.b64decode(encode_str(data))



from pathlib import Path

import click
from click_aliases import ClickAliasedGroup

from rtty_soda.cli import CliOptions, CliReader, CliWriter
from rtty_soda.formatters import FixedFormatter
from rtty_soda.services import EncodingService, EncryptionService, KeyService

__all__ = ["cli", "in_path"]

in_path = click.Path(
    exists=True,
    file_okay=True,
    dir_okay=False,
    readable=True,
    allow_dash=True,
    path_type=Path,
)

CONTEXT_SETTINGS = {"help_option_names": ["-h", "--help"]}


@click.group(cls=ClickAliasedGroup, context_settings=CONTEXT_SETTINGS)
@click.version_option(package_name="rtty-soda")
def cli() -> None:
    pass


@cli.command()  # pyright: ignore[reportAny]
@CliOptions.short_key_encoding
@CliOptions.output_file
@CliOptions.group_len
@CliOptions.line_len
@CliOptions.padding
@CliOptions.verbose
def genkey_cmd(
    encoding: str,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None:
    """Generate Private Key.

    Encoding: base26 | base31 | base36 | base64 | base94 | binary
    """
    formatter = FixedFormatter(group_len, line_len, padding)
    writer = CliWriter(output_file)
    service = KeyService(encoding, formatter, writer, verbose)
    service.genkey()


@cli.command()  # pyright: ignore[reportAny]
@click.argument("private_key_file", type=in_path)
@CliOptions.short_key_encoding
@CliOptions.output_file
@CliOptions.group_len
@CliOptions.line_len
@CliOptions.padding
@CliOptions.verbose
def pubkey_cmd(
    private_key_file: Path,
    encoding: str,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None:
    """Get Public Key.

    Encoding: base26 | base31 | base36 | base64 | base94 | binary
    """
    formatter = FixedFormatter(group_len, line_len, padding)
    writer = CliWriter(output_file)
    service = KeyService(encoding, formatter, writer, verbose)
    priv = CliReader(private_key_file)
    service.pubkey(priv)


@cli.command()  # pyright: ignore[reportAny]
@click.argument("password_file", type=in_path)
@CliOptions.short_key_encoding
@CliOptions.short_kdf_profile
@CliOptions.output_file
@CliOptions.group_len
@CliOptions.line_len
@CliOptions.padding
@CliOptions.verbose
def kdf_cmd(
    password_file: Path,
    encoding: str,
    profile: str,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None:
    """Key Derivation Function.

    Encoding: base26 | base31 | base36 | base64 | base94 | binary

    Profile: interactive | moderate | sensitive
    """
    formatter = FixedFormatter(group_len, line_len, padding)
    writer = CliWriter(output_file)
    service = KeyService(encoding, formatter, writer, verbose)
    password = CliReader(password_file)
    service.kdf(password=password, kdf_profile=profile)


@cli.command(aliases=["e"])  # pyright: ignore[reportAny]
@click.argument("private_key_file", type=in_path)
@click.argument("public_key_file", type=in_path)
@click.argument("message_file", type=in_path)
@CliOptions.text
@CliOptions.key_encoding
@CliOptions.data_encoding
@CliOptions.compression
@CliOptions.output_file
@CliOptions.group_len
@CliOptions.line_len
@CliOptions.padding
@CliOptions.verbose
def encrypt_public_cmd(
    private_key_file: Path,
    public_key_file: Path,
    message_file: Path,
    text: bool,
    key_encoding: str,
    data_encoding: str,
    compression: str,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None:
    """Encrypt Message (Public).

    Encoding: base26 | base31 | base36 | base64 | base94 | binary

    Compression: brotli | zstd | zlib | bz2 | lzma | raw
    """
    formatter = FixedFormatter(group_len, line_len, padding)
    writer = CliWriter(output_file)
    service = EncryptionService(
        text_mode=text,
        key_encoding=key_encoding,
        data_encoding=data_encoding,
        compression=compression,
        formatter=formatter,
        writer=writer,
        verbose=verbose,
    )
    priv = CliReader(private_key_file)
    pub = CliReader(public_key_file)
    message = CliReader(message_file)
    service.encrypt_public(private_key=priv, public_key=pub, message=message)


@cli.command(aliases=["es"])  # pyright: ignore[reportAny]
@click.argument("key_file", type=in_path)
@click.argument("message_file", type=in_path)
@CliOptions.text
@CliOptions.key_encoding
@CliOptions.data_encoding
@CliOptions.compression
@CliOptions.output_file
@CliOptions.group_len
@CliOptions.line_len
@CliOptions.padding
@CliOptions.verbose
def encrypt_secret_cmd(
    key_file: Path,
    message_file: Path,
    text: bool,
    key_encoding: str,
    data_encoding: str,
    compression: str,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None:
    """Encrypt Message (Secret).

    Encoding: base26 | base31 | base36 | base64 | base94 | binary

    Compression: brotli | zstd | zlib | bz2 | lzma | raw
    """
    formatter = FixedFormatter(group_len, line_len, padding)
    writer = CliWriter(output_file)
    service = EncryptionService(
        text_mode=text,
        key_encoding=key_encoding,
        data_encoding=data_encoding,
        compression=compression,
        formatter=formatter,
        writer=writer,
        verbose=verbose,
    )
    key = CliReader(key_file)
    message = CliReader(message_file)
    service.encrypt_secret(key, message)


@cli.command(aliases=["ep"])  # pyright: ignore[reportAny]
@click.argument("password_file", type=in_path)
@click.argument("message_file", type=in_path)
@CliOptions.text
@CliOptions.kdf_profile
@CliOptions.data_encoding
@CliOptions.compression
@CliOptions.output_file
@CliOptions.group_len
@CliOptions.line_len
@CliOptions.padding
@CliOptions.verbose
def encrypt_password_cmd(
    password_file: Path,
    message_file: Path,
    text: bool,
    kdf_profile: str,
    data_encoding: str,
    compression: str,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None:
    """Encrypt Message (Password).

    KDF profile: interactive | moderate | sensitive

    Encoding: base26 | base31 | base36 | base64 | base94 | binary

    Compression: brotli | zstd | zlib | bz2 | lzma | raw
    """
    formatter = FixedFormatter(group_len, line_len, padding)
    writer = CliWriter(output_file)
    service = EncryptionService(
        text_mode=text,
        key_encoding="binary",
        data_encoding=data_encoding,
        compression=compression,
        formatter=formatter,
        writer=writer,
        verbose=verbose,
    )
    pw = CliReader(password_file)
    message = CliReader(message_file)
    service.encrypt_password(password=pw, message=message, kdf_profile=kdf_profile)


@cli.command(aliases=["d"])  # pyright: ignore[reportAny]
@click.argument("private_key_file", type=in_path)
@click.argument("public_key_file", type=in_path)
@click.argument("message_file", type=in_path)
@CliOptions.text
@CliOptions.key_encoding
@CliOptions.data_encoding
@CliOptions.compression
@CliOptions.output_file
def decrypt_public_cmd(
    private_key_file: Path,
    public_key_file: Path,
    message_file: Path,
    text: bool,
    key_encoding: str,
    data_encoding: str,
    compression: str,
    output_file: Path | None,
) -> None:
    """Decrypt Message (Public).

    Encoding: base26 | base31 | base36 | base64 | base94 | binary

    Compression: brotli | zstd | zlib | bz2 | lzma | raw
    """
    writer = CliWriter(output_file)
    service = EncryptionService(
        text_mode=text,
        key_encoding=key_encoding,
        data_encoding=data_encoding,
        compression=compression,
        formatter=None,
        writer=writer,
        verbose=False,
    )
    priv = CliReader(private_key_file)
    pub = CliReader(public_key_file)
    message = CliReader(message_file)
    service.decrypt_public(private_key=priv, public_key=pub, message=message)


@cli.command(aliases=["ds"])  # pyright: ignore[reportAny]
@click.argument("key_file", type=in_path)
@click.argument("message_file", type=in_path)
@CliOptions.text
@CliOptions.key_encoding
@CliOptions.data_encoding
@CliOptions.compression
@CliOptions.output_file
def decrypt_secret_cmd(
    key_file: Path,
    message_file: Path,
    text: bool,
    key_encoding: str,
    data_encoding: str,
    compression: str,
    output_file: Path | None,
) -> None:
    """Decrypt Message (Secret).

    Encoding: base26 | base31 | base36 | base64 | base94 | binary

    Compression: brotli | zstd | zlib | bz2 | lzma | raw
    """
    writer = CliWriter(output_file)
    service = EncryptionService(
        text_mode=text,
        key_encoding=key_encoding,
        data_encoding=data_encoding,
        compression=compression,
        formatter=None,
        writer=writer,
        verbose=False,
    )
    key = CliReader(key_file)
    message = CliReader(message_file)
    service.decrypt_secret(key, message)


@cli.command(aliases=["dp"])  # pyright: ignore[reportAny]
@click.argument("password_file", type=in_path)
@click.argument("message_file", type=in_path)
@CliOptions.text
@CliOptions.kdf_profile
@CliOptions.data_encoding
@CliOptions.compression
@CliOptions.output_file
def decrypt_password_cmd(
    password_file: Path,
    message_file: Path,
    text: bool,
    kdf_profile: str,
    data_encoding: str,
    compression: str,
    output_file: Path | None,
) -> None:
    """Decrypt Message (Password).

    KDF profile: interactive | moderate | sensitive

    Encoding: base26 | base31 | base36 | base64 | base94 | binary

    Compression: brotli | zstd | zlib | bz2 | lzma | raw
    """
    writer = CliWriter(output_file)
    service = EncryptionService(
        text_mode=text,
        key_encoding="binary",
        data_encoding=data_encoding,
        compression=compression,
        formatter=None,
        writer=writer,
        verbose=False,
    )
    pw = CliReader(password_file)
    message = CliReader(message_file)
    service.decrypt_password(password=pw, message=message, kdf_profile=kdf_profile)


@cli.command()  # pyright: ignore[reportAny]
@click.argument("in_encoding")
@click.argument("out_encoding")
@click.argument("file", type=in_path)
@CliOptions.output_file
@CliOptions.group_len
@CliOptions.line_len
@CliOptions.padding
@CliOptions.verbose
def encode_cmd(
    in_encoding: str,
    out_encoding: str,
    file: Path,
    output_file: Path | None,
    group_len: int,
    line_len: int,
    padding: int,
    verbose: bool,
) -> None:
    """Encode File.

    Encoding: base26 | base31 | base36 | base64 | base94 | binary
    """
    writer = CliWriter(output_file)
    formatter = FixedFormatter(group_len, line_len, padding)
    service = EncodingService(
        in_encoding=in_encoding,
        out_encoding=out_encoding,
        formatter=formatter,
        writer=writer,
        verbose=verbose,
    )
    data = CliReader(file)
    service.encode(data)


if __name__ == "__main__":
    cli()



from .base26_encoder import Base26Encoder
from .base31_encoder import Base31Encoder
from .base36_encoder import Base36Encoder
from .base64_encoder import Base64Encoder
from .base94_encoder import Base94Encoder
from .encoder import Encoder
from .functions import decode_bytes, encode_str
from .scsu import scsu_decode, scsu_encode

__all__ = [
    "ENCODERS",
    "Base26Encoder",
    "Base31Encoder",
    "Base36Encoder",
    "Base64Encoder",
    "Base94Encoder",
    "Encoder",
    "decode_bytes",
    "encode_str",
    "scsu_decode",
    "scsu_encode",
]

ENCODERS: dict[str, Encoder] = {
    "base26": Base26Encoder(),
    "base31": Base31Encoder(),
    "base36": Base36Encoder(),
    "base64": Base64Encoder(),
    "base94": Base94Encoder(),
}



from typing import TYPE_CHECKING

from nacl.public import PrivateKey

from rtty_soda.cryptography import kdf
from rtty_soda.encoders import ENCODERS, encode_str

from .service import Service

if TYPE_CHECKING:
    from rtty_soda.formatters import Formatter
    from rtty_soda.interfaces import Reader, Writer

__all__ = ["KeyService"]


class KeyService(Service):
    def __init__(
        self, encoding: str, formatter: Formatter, writer: Writer, verbose: bool
    ) -> None:
        super().__init__(formatter, writer, verbose)
        self.encoder = ENCODERS.get(encoding)

    def genkey(self) -> None:
        key = bytes(PrivateKey.generate())
        self.write_output(key, self.encoder)

    def pubkey(self, priv_key: Reader) -> None:
        priv_bytes = self.read_input(priv_key, self.encoder)
        priv = PrivateKey(priv_bytes)
        pub = bytes(priv.public_key)
        self.write_output(pub, self.encoder)

    @staticmethod
    def derive_key(password: Reader, kdf_profile: str) -> bytes:
        prof = kdf.KDF_PROFILES[kdf_profile]
        pw_str = password.read_str().strip()
        pw_bytes = encode_str(pw_str)
        return kdf.kdf(password=pw_bytes, profile=prof)

    def kdf(self, password: Reader, kdf_profile: str) -> None:
        key = self.derive_key(password, kdf_profile)
        self.write_output(key, self.encoder)
